'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.sign = exports.register = exports.ensureSupport = exports.isSupported = exports.ErrorNames = exports.ErrorCodes = void 0;
// @ts-ignore
const generated_google_u2f_api_1 = require("./generated-google-u2f-api");
const types_1 = require("./types");
Object.defineProperty(exports, "ErrorCodes", { enumerable: true, get: function () { return types_1.ErrorCodes; } });
Object.defineProperty(exports, "ErrorNames", { enumerable: true, get: function () { return types_1.ErrorNames; } });
// Feature detection (yes really)
// For IE and Edge detection, see https://stackoverflow.com/questions/31757852#31757969
// and https://stackoverflow.com/questions/56360225#56361977
const isBrowser = (typeof navigator !== 'undefined') && !!navigator.userAgent;
const isSafari = isBrowser
    && navigator.userAgent.match(/Safari\//)
    && !navigator.userAgent.match(/Chrome\//);
const isSupportedSafari = isBrowser
    && navigator.userAgent.match(/Safari\/(1[3456789])/)
    && !navigator.userAgent.match(/Chrome\//);
const isEDGE = isBrowser && /(Edge\/)|(edg\/)/i.test(navigator.userAgent);
const isIE = isBrowser && /(MSIE 9|MSIE 10|rv:11.0)/i.test(navigator.userAgent);
var _backend = null;
function getBackend() {
    if (_backend)
        return _backend;
    const supportChecker = new Promise(function (resolve, reject) {
        function notSupported() {
            resolve({ u2f: null });
        }
        if (!isBrowser)
            return notSupported();
        if (isSafari && !isSupportedSafari)
            // Safari doesn't support U2F, and the Safari-FIDO-U2F
            // extension lacks full support (Multi-facet apps), so we
            // block it until proper support.
            return notSupported();
        const hasNativeSupport = (typeof window.u2f !== 'undefined') &&
            (typeof window.u2f.sign === 'function');
        if (hasNativeSupport)
            return resolve({ u2f: window.u2f });
        if (isEDGE || isIE)
            // We don't want to check for Google's extension hack on EDGE & IE
            // as it'll cause trouble (popups, etc)
            return notSupported();
        if (location.protocol === 'http:')
            // U2F isn't supported over http, only https
            return notSupported();
        if (typeof MessageChannel === 'undefined')
            // Unsupported browser, the chrome hack would throw
            return notSupported();
        // Test for google extension support
        generated_google_u2f_api_1.chromeApi.isSupported(function (ok) {
            if (ok)
                resolve({ u2f: generated_google_u2f_api_1.chromeApi });
            else
                notSupported();
        });
    })
        .then(function (response) {
        _backend = response.u2f ? supportChecker : null;
        return response;
    });
    return supportChecker;
}
function isSupported() {
    return getBackend()
        .then(backend => !!backend.u2f);
}
exports.isSupported = isSupported;
function _ensureSupport(backend) {
    if (!backend.u2f) {
        if (location.protocol === 'http:')
            throw new Error("U2F isn't supported over http, only https");
        throw new Error("U2F not supported");
    }
}
function ensureSupport() {
    return getBackend()
        .then(_ensureSupport);
}
exports.ensureSupport = ensureSupport;
function arrayify(value) {
    if (value != null && Array.isArray(value))
        return value;
    return value == null
        ? []
        : Array.isArray(value)
            ? [...value]
            : [value];
}
function register(registerRequests, signRequests, timeout) {
    const _registerRequests = arrayify(registerRequests);
    if (typeof signRequests === 'number' && typeof timeout === 'undefined') {
        timeout = signRequests;
        signRequests = [];
    }
    const _signRequests = arrayify(signRequests);
    return getBackend()
        .then(function (backend) {
        _ensureSupport(backend);
        const { u2f } = backend;
        return new Promise(function (resolve, reject) {
            function callback(response) {
                if (response.errorCode)
                    reject(types_1.makeError("Registration failed", response));
                else {
                    delete response.errorCode;
                    resolve(response);
                }
            }
            const appId = _registerRequests[0].appId;
            u2f.register(appId, _registerRequests, _signRequests, callback, timeout);
        });
    });
}
exports.register = register;
function sign(signRequests, timeout) {
    const _signRequests = arrayify(signRequests);
    return getBackend()
        .then(function (backend) {
        _ensureSupport(backend);
        const { u2f } = backend;
        return new Promise(function (resolve, reject) {
            function callback(response) {
                if (response.errorCode)
                    reject(types_1.makeError("Sign failed", response));
                else {
                    delete response.errorCode;
                    resolve(response);
                }
            }
            const appId = _signRequests[0].appId;
            const challenge = _signRequests[0].challenge;
            const registeredKeys = [].concat(..._signRequests
                .map(({ version, keyHandle, appId }) => arrayify(keyHandle)
                .map(keyHandle => ({ version, keyHandle, appId }))));
            u2f.sign(appId, challenge, registeredKeys, callback, timeout);
        });
    });
}
exports.sign = sign;
